<?php

declare(strict_types=1);

namespace apexl\Io\interfaces;

use apexl\Vault\interfaces\driver;

interface QueryDecorator
{
    public function decorate(RelationshipInterface $relationship, driver $query): driver;
}
