<?php

declare(strict_types=1);

namespace apexl\Io\modules\system\interfaces;

use apexl\Io\modules\system\exceptions\ModuleFinderException;

interface ModuleFinderInterface
{
    /**
     * @throws ModuleFinderException
     */
    public function all(): array;

    /**
     * @throws ModuleFinderException
     */
    public function installed(): array;

}