<?php

declare(strict_types=1);

$environment = env('APP_ENVIRONMENT', 'production');

return [
    'reporting' => env('ERROR_REPORTING', E_ALL ^ E_DEPRECATED ^ E_USER_DEPRECATED),
    'display' => env('ERROR_DISPLAY', $environment !== 'production'),
];