<?php

declare(strict_types=1);

return [
    'streams' => [
        "default" => [
            "level" => env('LOG__DEFAULT_LEVEL'),
            "handlers" => [
                [
                    "destination" => "file"
                ]
            ]
        ],
        "query" => [
            "level" => env('LOG__QUERY_LEVEL'),
            "handlers" => [
                [
                    "destination" => "file"
                ]
            ]
        ],
        "error" => [
            "level" => env('LOG__ERROR_LEVEL'),
            "handlers" => [
                [
                    "destination" => "error_log"
                ],
                [
                    "destination" => "file",
                    "file" => "error.log"
                ]
            ]
        ],

    ]
];